import { ROLE_NAME } from "@/constant";
import {
  CloudUploadOutlined,
  DatabaseOutlined,
  UnorderedListOutlined,
} from "@ant-design/icons";
import { MenuProps } from "antd";

export type MenuItem = Required<MenuProps>["items"][number];

const getItem = (
  label: React.ReactNode,
  key: React.Key,
  icon?: React.ReactNode,
  children?: MenuItem[],
  type?: "group"
): MenuItem => {
  return {
    key,
    icon,
    children,
    label,
    type,
  } as MenuItem;
};

const adminMenuItems: MenuItem[] = [
  getItem("影像", "dicom", <DatabaseOutlined />, [
    getItem("上传", "/upload", <CloudUploadOutlined />),
    getItem("列表", "/list", <UnorderedListOutlined />),
  ]),
];

const annotatorMenuItems: MenuItem[] = [
  getItem("影像", "dicom", <DatabaseOutlined />, [
    getItem("标注", "/list", <CloudUploadOutlined />),
  ]),
];

/**
 * key对应roles的name字段，创建角色的key
 */
export const roleMenusMapping: Record<ROLE_NAME, MenuItem[]> = {
  [ROLE_NAME.ADMIN]: adminMenuItems,
  [ROLE_NAME.ANNOTATOR]: annotatorMenuItems,
};
